import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * The player controlled spaceship.
 * 
 * @author Jacob Moss
 * @version 5.0
 */
public class Voyager extends Ship_Stuff
{
    //finals
    private static final int reloadTime = 15;
    private static final int phaserReloadTime = 10;
    private static final int damageMax = 99;
    //integers
    public int torpedoCount = 30;
    public int score;
    private int level = 1;
    public int speed = 4;
    public int invspeed = 4;
    public int dilithiumStore = 20;
    private int reloadDelayCount;
    private int phaserDelayCount;
    public int damageLevel;
    public int torpedoDamage = 30;
    public int phaserDamage = 10;
    private int delay = 0;
    //counters
    public Counter counterTorpedo;
    private Counter counterScore;
    private Counter temp;
    private Counter counterLevel;
    public Counter counterSpeed;
    public Counter counterDilithiumStore;

    /**
     * sets some of the integers
     */
    public Voyager()
    {
        reloadDelayCount = 10;
        phaserDelayCount = 0;
        damageLevel = 0;
        score = 0;
    }

    /**
     * defines each counter
     */
    public void addedToWorld(World world)
    {
        counterTorpedo = new Counter("Torpedoes: ");
        counterTorpedo.setValue(torpedoCount);
        getWorld().addObject(counterTorpedo, 148, 484);

        counterScore = new Counter("Score: ");
        counterScore.setValue(score);
        getWorld().addObject(counterScore, 268, 484);

        temp = new Counter("Health: ");
        temp.setValue(damageLevel);
        //getWorld().addObject(temp, 380, 483);

        counterLevel = new Counter("Level: ");
        counterLevel.setValue(level);
        getWorld().addObject(counterLevel, 373, 484);

        counterSpeed = new Counter("Speed: ");
        counterSpeed.setValue(speed);
        getWorld().addObject(counterSpeed, 480, 484);

        counterDilithiumStore = new Counter("Dilithium: ");
        counterDilithiumStore.setValue(dilithiumStore);
        getWorld().addObject(counterDilithiumStore, 590, 484);
    }

    /**
     * links to methods and constantly increases the delay counts for the weapons
     */
    public void act() 
    {
        reloadDelayCount++;
        phaserDelayCount = phaserDelayCount+1;
        delay++;

        checkKeys();
        checkCollision();
        gameOver();
        nextLevel();
    }    

    /**
     * increments and sets the value of score
     */
    public void score()
    {
        score++;
        counterScore.setValue(score);
    }

    /**
     * checking which keys are pressed and does stuff
     */
    private void checkKeys()
    {
        thrust( Greenfoot.isKeyDown("w") || Greenfoot.isKeyDown("up") );
        if( Greenfoot.isKeyDown("s") ) {
            setRotation(getRotation());
            move(-4);
        }
        if( Greenfoot.isKeyDown("down") ) {
            setRotation(getRotation());
            move(-4);
        }
        if( Greenfoot.isKeyDown("a") ) {
            turn(-invspeed);
        }
        if( Greenfoot.isKeyDown("left") ) {
            turn(-invspeed);
        }
        if( Greenfoot.isKeyDown("d") ) {
            turn(invspeed);
        }
        if( Greenfoot.isKeyDown("right") ) {
            turn(invspeed);
        }
        if( Greenfoot.isKeyDown("space") || Greenfoot.isKeyDown("c") ) {
            ((Space1)getWorld()).fire(false);
        }
        if( Greenfoot.isKeyDown("e") ) {
            ((Space1)getWorld()).fire(true);
        }
    }

    /**
     * changes the image of the ship and moves the ship at the speed defined in a field at the top
     */
    private void thrust ( boolean thrustOn )
    {
        if (thrustOn) {
            setImage("rocketEngaged.png");
            setRotation(getRotation());
            move(speed);
        }
        if (!thrustOn) {
            setImage("rocket.png");
        }
    }

    /**
     * third and final firing method, linked from the world which is linked from buttonFirePhaser and buttonFireTorpedo
     */
    public void fire ( boolean isTorpedo )
    {
        if ( isTorpedo ) {
            if( torpedoCount >0 ) {
                if (reloadDelayCount >= reloadTime) {
                    Torpedo torpedo = new Torpedo(torpedoDamage);
                    getWorld().addObject(torpedo, getX(), getY());
                    torpedo.setRotation(getRotation());
                    reloadDelayCount = 0;
                    torpedoCount--;
                    counterTorpedo.setValue(torpedoCount);
                    Greenfoot.playSound("Torpedo.mp3");
                }
            }
        }
        else {
            if (phaserDelayCount >= phaserReloadTime) {
                Phaser phaser = new Phaser(phaserDamage);
                getWorld().addObject(phaser, getX(), getY());
                phaser.setRotation(getRotation());
                phaserDelayCount = 0;
                Greenfoot.playSound("Phaser.wav");
            }
        }
    }

    /**
     * checks if the ship has collided with something and does stuff as a result
     */
    private void checkCollision()
    {
        if (getX() < 115) {
            setLocation(115, getY());
        }
        if (getY() < 97) {
            setLocation(getX(), 97);
        }
        if( getY() > 446 ) {
            setLocation(getX(), 446);
        }

        Actor asteroid = getOneIntersectingObject(Asteroid.class);
        if (asteroid!= null) {
            dilithiumStore = dilithiumStore+5;
            counterDilithiumStore.setValue(dilithiumStore);
        }

        enemyPulse enemPulse = (enemyPulse) getOneIntersectingObject (enemyPulse.class);
        if (enemPulse!= null) {
            damageLevel = damageLevel+10;
            getWorld().removeObject(enemPulse);
            Shield shield = new Shield();
            getWorld().addObject(shield, getX(), getY());
            getWorld().addObject(new debrisSmall(), getX(), getY());
            shield.setRotation(getRotation());
        }
        
        enemyTorpedo enemTorpedo = (enemyTorpedo) getOneIntersectingObject (enemyTorpedo.class);
        if (enemTorpedo!= null) {
            damageLevel = damageLevel+20;
            getWorld().removeObject(enemTorpedo);
            Shield shield = new Shield();
            getWorld().addObject(shield, getX(), getY());
            shield.setRotation(getRotation());
            getWorld().addObject(new debrisSmall(), getX(), getY());
        }

        Actor torpedoRestock = getOneIntersectingObject(TorpedoPack.class);
        {
            if(torpedoRestock!= null)
            {
                torpedoCount = torpedoCount+2;
                counterTorpedo.setValue(torpedoCount);
            }
        }

        Actor damageInc = getOneIntersectingObject(HealthPack.class);
        {
            if(damageInc!= null) {
                if( damageLevel >0 ) {
                    damageLevel = damageLevel-10;
                }
            }
        }
    }

    /**
     * checks if the damagelevel is at the maximum and stops the game and adds the gameover screen
     */
    private void gameOver()
    {
        if (damageLevel >= damageMax)
        {
            Greenfoot.stop();
            getWorld().addObject(new gameOver(), 450, 250);
        }
    }

    /**
     * check whether to change the level and make the game harder.
     */
    private void nextLevel()
    {
        if( score >= 10 ) {
            level = 2;
            counterLevel.setValue(level);
            ((Space1)getWorld()).level2();
        }
        if( score >= 20 ) {
            level = 3;
            counterLevel.setValue(level);
            ((Space1)getWorld()).level3();
        }
    }

    /**
     * linked from buttonRepair and decreases the damagelevel
     */
    public void repair()
    {
        if(damageLevel >= 10) {
            damageLevel = damageLevel-10;
        }
    }

    /**
     * linked from some buttons and decreases the dilithium
     */
    public void dilithiumStoreDecrease()
    {
        dilithiumStore = dilithiumStore-10;
        counterDilithiumStore.setValue(dilithiumStore);
    }
}
